#ifndef Q_COMMAND
#define Q_COMMAND

#include "qLab/qError.lsl"

list qCommandCreate(string commandName)
{
  list command;

  command = [ commandName ];

  return command;
}



list qCommandAddSwitch(list command, list switch)
{
  string switchName    = llList2String(switch, 0);
  integer settingCount = llGetListLength(switch) - 1;

  command += [switchName, settingCount];

  if (settingCount > 0)
    command += llList2List(switch, 1, settingCount);

  return command;
}


qCommandUsage(list command)
{
  qUserError("USAGE: " + llList2String(command, 0));

  integer commandLength = llGetListLength(command);

  integer i;
  for (i = 1; i < commandLength; ) // i incremented below
    {
      string output = "";

      string switch         = llList2String(command, i);
      i++;
      integer settingCount  = llList2Integer(command, i);
      i++;

      output = "     " + switch;

      integer j;
      for (j = 0; j < settingCount; j++)
	{
	  output = output + " " + llList2String(command, i);
	  i++;
	}

      qUserError(output);
    }
}




list qCommandParseText(list command, string text)
{
  integer commandLength = llGetListLength(command);

  list parse = llParseString2List(text, [" "], []);
  integer parseLength = llGetListLength(parse);

  // make sure this is our command
  string parseCommand   = llList2String(parse, 0);
  string commandCommand = llList2String(command, 0);

  string objectName = llGetObjectName();
  string objectKey  = (string)llGetKey();

  if ((parseCommand != "*") &&   // wildcard matches all commands
      (parseCommand != commandCommand) &&
      (parseCommand != objectName + "/" + commandCommand) &&
      (parseCommand != objectKey + "/" + commandCommand))
    return [];

  list switches = [ parseCommand ];;

  integer i;
  for (i = 1; i < parseLength;) // i incremented below
    {
      string parseWord = llList2String(parse, i);
      i++;

      integer found = 0;

      // scan switch list to see which one we've got:
      integer j;
      for (j = 1; (j < commandLength) && (found == 0); ) // j incremented below
	{

	  string switch         = llList2String(command, j);
	  j++;
	  integer settingCount  = llList2Integer(command, j);
	  j++;

	  if (parseWord == switch)
	    {
	      found = 1;
	      switches += [ switch ];

	      integer k;
	      for (k = 0; k < settingCount; k++)
		{
		  if (i >= parseLength)
		    {
		      qUserError("\"" + switch + "\" requires " +
				 (string)settingCount + " options.");
		      qCommandUsage(command);
		      return [];
		    }

		  string parseWord = llList2String(parse, i);
		  i++;

		  string settingType = llList2String(command, j+k);

		  if (settingType == "float")
		    switches += [(float) parseWord];

		  else if (settingType == "integer")
		    switches += [(integer) parseWord];

		  else // assume string
		    switches += [ parseWord ];
		}
	    }
	  
	  j += settingCount;
	}


      if (!found)
	{
	  qUserError("\"" + parseWord + "\" not understood.");
	  qCommandUsage(command);
	  return [];
	}
    }


  return switches;
}

// universal switches

#define Q_COMMAND_DEBUG_SWITCH "-debug"
#define Q_COMMAND_RESET_SWITCH "-reset"
#define Q_COMMAND_DIE_SWITCH   "-die"
#define Q_COMMAND_HELP_SWITCH  "-help"

list qCommandAddDebugSwitch(list command)
{
  return qCommandAddSwitch(command, [Q_COMMAND_DEBUG_SWITCH, "integer"]);
}

list qCommandAddResetSwitch(list command)
{
  return qCommandAddSwitch(command, [Q_COMMAND_RESET_SWITCH]);
}

list qCommandAddDieSwitch(list command)
{
  return qCommandAddSwitch(command, [Q_COMMAND_DIE_SWITCH]);
}

list qCommandAddHelpSwitch(list command)
{
  return qCommandAddSwitch(command, [Q_COMMAND_HELP_SWITCH]);
}


list qCommandAddUniversalSwitches(list command)
{
  command = qCommandAddDebugSwitch(command);
  command = qCommandAddResetSwitch(command);
  command = qCommandAddDieSwitch(command);
  command = qCommandAddHelpSwitch(command);

  return command;
}

integer qCommandExecuteUniversalSwitch(list switches, integer i)
{
  string switch = llList2String(switches, i);

  if (switch == Q_COMMAND_DEBUG_SWITCH)
    {
      i++;
      qDebugSetLevel(llList2Integer(switches, i));
      i++;
    }
  else if (switch == Q_COMMAND_RESET_SWITCH)
    {
      llResetScript();
      i++; // not really necessary
    }
  else if (switch == Q_COMMAND_DIE_SWITCH)
    {
      llDie();
      i++; // not really necessary
    }

  else if (switch == Q_COMMAND_HELP_SWITCH)
    {
      llGiveInventory(llGetOwner(), 	
		      "Instruction Manual");
      i++; // not really necessary
    }

  return i;
}



#endif
