#ifndef Q_COMMAND_PRIM
#define Q_COMMAND_PRIM

#include "qLab/qError.lsl"
#include "qLab/qCommand.lsl"
#include "qLab/qPosition.lsl"


#define Q_PRIM_SETGLOBALPOS    "-setGlobalPos"
#define Q_PRIM_SETRELATIVEPOS  "-setRelativePos"
#define Q_PRIM_SETSCALE        "-setScale"
#define Q_PRIM_SETROT          "-setRot"
#define Q_PRIM_SETCOLOR        "-setColor"
#define Q_PRIM_SETALPHA        "-setAlpha"
#define Q_PRIM_SETTEXTURE      "-setTexture"
#define Q_PRIM_SETTEXTUREPOS   "-setTexturePos"
#define Q_PRIM_SETTYPE         "-setType"
#define Q_PRIM_DELETESCRIPT    "-deleteScript"
#define Q_PRIM_SETOBJECTNAME   "-setObjectName"
#define Q_PRIM_SETOBJECTDESC   "-setObjectDesc"
#define Q_PRIM_SLEEP           "-sleep"
#define Q_PRIM_GOTOGLOBALPOS   "-gotoGlobalPos"
#define Q_PRIM_GOTORELATIVEPOS "-gotoRelativePos"
#define Q_PRIM_GOTOSIM         "-gotoSim"



integer qCommandPrimExecuteSwitch(list switches, integer i)
{
  qDebug("qCommandExecutePrimSwitch", 18);

  string switch = llList2String(switches, i);

  if (switch == Q_PRIM_SETGLOBALPOS)
    {
      qSetGlobalPosition(<llList2Float(switches, i+1),
			 llList2Float(switches, i+2),
			 llList2Float(switches, i+3)>);
      i+=4;
    }
  else if (switch == Q_PRIM_SETRELATIVEPOS)
    {
      vector position = qGetGlobalPosition();

      qSetGlobalPosition(position + (<llList2Float(switches, i+1),
				     llList2Float(switches, i+2),
				     llList2Float(switches, i+3)>));
      i+=4;
    }
  else if (switch == Q_PRIM_SETSCALE)
    {
      llSetScale(<llList2Float(switches, i+1),
		 llList2Float(switches, i+2),
		 llList2Float(switches, i+3)>);
      i+=4;
    }
  else if (switch == Q_PRIM_SETROT)
    {
      llSetRot(llEuler2Rot(<llList2Float(switches, i+1),
			   llList2Float(switches, i+2),
			   llList2Float(switches, i+3)>));
      i+=4;
    }

  else if (switch == Q_PRIM_SETCOLOR)
    {
      llSetColor(<llList2Float(switches, i+1),
		 llList2Float(switches, i+2),
		 llList2Float(switches, i+3)>, 
		 llList2Integer(switches, i+4));
      i+=5;
    }
  
  else if (switch == Q_PRIM_SETALPHA)
    {
      llSetAlpha(llList2Float(switches, i+1),
		 llList2Integer(switches, i+2));
      i+=3;
    }

  else if (switch == Q_PRIM_SETTEXTURE)
    {
      string texture = llList2String(switches, i+1);

      if (texture == "blank")
	texture = "5748decc-f629-461c-9a36-a35a221fe21f";

      llSetTexture(texture,
		   llList2Integer(switches, i+2));
      i+=3;
    }

  else if (switch == Q_PRIM_SETTEXTUREPOS)
    {
      integer side = llList2Integer(switches, i+6);

      llOffsetTexture(llList2Float(switches, i+1),
		      llList2Float(switches, i+2),
		      side);
      llRotateTexture(llList2Float(switches, i+3),
		      side);
      llScaleTexture(llList2Float(switches, i+4),
		     llList2Float(switches, i+5),
		     side);

      i+=7;
    }

  else if (switch == Q_PRIM_SETTYPE)
    {
      integer type = llList2Integer(switches, i+1);
      
      if (type == 0)
	llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_BOX, 0, <0.0, 1.0, 0.0>, 
			      0.0, <0.0, 0.0, 0.0>, <1.0, 1.0, 0.0>, <0.0, 0.0, 0.0>]);
      else if (type == 1)
	llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_CYLINDER, 0, <0.0, 1.0, 0.0>, 0.0, 
			      <0.0, 0.0, 0.0>, <1.0, 1.0, 0.0>, <0.0, 0.0, 0.0>]);

      else if (type == 2)
	llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_PRISM, 0, <0.0, 1.0, 0.0>, 0.0, 
			      <0.0, 0.0, 0.0>, <0.0, 0.0, 0.0>, <0.0, 0.0, 0.0>]);

      else if (type == 3)
	llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_SPHERE, 0, <0.0, 1.0, 0.0>, 0.0, 
			      <0.0, 0.0, 0.0>, <0.0, 1.0, 0.0>]);

      else if (type == 4)
	llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_TORUS, 0, <0.0, 1.0, 0.0>, 0.0, 
			      <0.0, 0.0, 0.0>, <1.0, 0.25, 0.0>, <0.0, 0.0, 0.0>, 
			      <0.0, 1.0, 0.0>, <0.0, 0.0, 0.0>, 1.0, 0.0, 0.0]);

      else if (type == 5)
	llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_TUBE, 0, <0.0, 1.0, 0.0>, 0.0, 
			      <0.0, 0.0, 0.0>, <1.0, 0.25, 0.0>, <0.0, 0.0, 0.0>, 
			      <0.0, 1.0, 0.0>, <0.0, 0.0, 0.0>, 1.0, 0.0, 0.0]);

      else if (type == 6)
	llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_RING, 0, <0.0, 1.0, 0.0>, 0.0, 
			      <0.0, 0.0, 0.0>, <1.0, 0.25, 0.0>, <0.0, 0.0, 0.0>, 
			      <0.0, 1.0, 0.0>, <0.0, 0.0, 0.0>, 1.0, 0.0, 0.0]);
			     
	

      i+=2;
    }

  else if (switch == Q_PRIM_DELETESCRIPT)
    {
      llRemoveInventory(llGetScriptName());
      i++;
    }

  else if (switch == Q_PRIM_SETOBJECTNAME)
    {
      llSetObjectName(llList2String(switches, i+1));
      i+=2;
    }

  else if (switch == Q_PRIM_SETOBJECTDESC)
    {
      llSetObjectDesc(llList2String(switches, i+1));
      i+=2;
    }

  else if (switch == Q_PRIM_SLEEP)
    {
      llSleep(llList2Integer(switches, i+1));
      i+=2;
    }

  else if (switch == Q_PRIM_GOTOSIM)
    {
      vector position = qGetGlobalPosition();
      position.x = llList2Float(switches, i+1) * 256 + 128;
      position.y = llList2Float(switches, i+2) * 256 + 128;
      qGotoGlobalPosition(position);

      i+=3;
    }

  else if (switch == Q_PRIM_GOTOGLOBALPOS)
    {
      qGotoGlobalPosition(<llList2Float(switches, i+1),
			  llList2Float(switches, i+2),
			  llList2Float(switches, i+3)>);
      i+=4;
    }
  else if (switch == Q_PRIM_GOTORELATIVEPOS)
    {
      vector position = qGetGlobalPosition();
      qGotoGlobalPosition(position + (<llList2Float(switches, i+1),
				      llList2Float(switches, i+2),
				      llList2Float(switches, i+3)>));
      i+=4;
    }


  else if (switch == Q_PRIM_GOTOSIM)
    {
      vector position = qGetGlobalPosition();
      position.x = llList2Float(switches, i+1) * 256 + 128;
      position.y = llList2Float(switches, i+2) * 256 + 128;
      qGotoGlobalPosition(position);

      i+=3;
    }

  return i;
}




list qCommandAddPrimSwitches(list command)
{
  command = qCommandAddSwitch(command, [Q_PRIM_SETGLOBALPOS, 
					"float", "float", "float"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETRELATIVEPOS, 
					"float", "float", "float"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETSCALE, 
					"float", "float", "float"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETROT, 
					"float", "float", "float"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETCOLOR, 
					"float", "float", "float", "integer"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETALPHA, 
					"float", "integer"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETTEXTURE, 
					"string", "integer"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETTEXTUREPOS, 
					"float", "float", "float", "float", "float", "integer"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETTYPE, 
					"integer"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETOBJECTNAME, 
					"string"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SETOBJECTDESC, 
					"string"]);
  command = qCommandAddSwitch(command, [Q_PRIM_SLEEP, 
					"integer"]);
  command = qCommandAddSwitch(command, [Q_PRIM_DELETESCRIPT]);

  command = qCommandAddSwitch(command, [Q_PRIM_GOTOGLOBALPOS, 
					"float", "float", "float"]);
  command = qCommandAddSwitch(command, [Q_PRIM_GOTORELATIVEPOS, 
					"float", "float", "float"]);
  command = qCommandAddSwitch(command, [Q_PRIM_GOTOSIM, 
					"integer", "integer"]);



  return command;
}


#endif // Q_COMMAND_PRIM
