#include "qLab/qError.lsl"
#include "qLab/qCommand.lsl"
#include "qLab/qDynaPrim/qDynaPrim.lsl"


list command;
key scriptKey;
integer scriptLineNumber;
integer scriptRunning;
integer scriptCurrentPrim;

#define Q_DYNASCRIPT_COMMAND_CHANNEL 500

#define Q_DYNASCRIPT_SCRIPT_SWITCH "-script"


executeScriptLine(string line)
{
  qDebug("executeScriptLine: " + line, 10);
  list words = llParseString2List(line, [" "], []);

  if ((llGetListLength(words) == 1) &&
      (llList2String(words, 0) == "newPrim"))
    scriptCurrentPrim = qDynaRez();
  
  else
    llSay(scriptCurrentPrim, line);
}


startScript(key script)
{
  qDebug("startScript: " + (string)script, 5);
  scriptRunning = 1;
  scriptKey = script;
  scriptLineNumber = 0;
  
  llGetNotecardLine(scriptKey, scriptLineNumber);
}


endScript()
{
  qDebug("endScript", 5);
  scriptRunning = 0;
  qDynaCleanup();
}



buildCommand()
{
  command = qCommandCreate("qDynaScript");
  command = qCommandAddSwitch(command, [Q_DYNASCRIPT_SCRIPT_SWITCH, "string"]);
  command = qCommandAddUniversalSwitches(command);
}


initCommand()
{
  buildCommand();
  
  llListen(Q_DYNASCRIPT_COMMAND_CHANNEL, "", NULL_KEY, "");
}



executeCommand(list switches)
{
  integer switchesLength = llGetListLength(switches);

  integer i;
  for (i = 1; i < switchesLength; )
    {
      string switch = llList2String(switches, i);

      if (switch == Q_DYNASCRIPT_SCRIPT_SWITCH)
        {
	  startScript((key)llList2String(switches, i+1));
          i += 2;
        }
      else
        i = qCommandExecuteUniversalSwitch(switches, i);

    }
}


listenCommand(integer channel, string name, key id,  string message)
{
  // we only accept commands from our owner
  if (id != llGetOwner())
    return;

  list switches = qCommandParseText(command, message);

  if (switches != [])
    executeCommand(switches);
}



default
{
  state_entry()
    {
      scriptRunning = 0;
      initCommand();
    }

  dataserver(key id, string data)
    {
      if (data != EOF)
        {
	  executeScriptLine(data);

          scriptLineNumber++;
          llGetNotecardLine(scriptKey, scriptLineNumber);
        }
      else
        endScript();
    }


  listen(integer channel, string name, key id,  string message)
    {
      if(!scriptRunning)
	listenCommand(channel, name, id, message);
    }

}
