#include "qLab/qCommand.lsl"
#include "qLab/qPosition.lsl"

#define TEXTURE_COMMAND_CHANNEL 500

#define Q_TEXTURE_SETCOLOR        "-setColor"
#define Q_TEXTURE_SETALPHA        "-setAlpha"
#define Q_TEXTURE_SETTEXTURE      "-setTexture"
#define Q_TEXTURE_SETTEXTUREPOS   "-setTexturePos"
#define Q_TEXTURE_DELETESCRIPT    "-deleteScript"
#define Q_TEXTURE_SLEEP           "-sleep"


list textureCommand;

buildTextureCommand()
{
  textureCommand = qCommandCreate("texture");
  textureCommand = qCommandAddUniversalSwitches(textureCommand);

  textureCommand = qCommandAddSwitch(textureCommand, [Q_TEXTURE_SETCOLOR, 
                                        "float", "float", "float", "integer"]);
  textureCommand = qCommandAddSwitch(textureCommand, [Q_TEXTURE_SETALPHA, 
                                        "float", "integer"]);
  textureCommand = qCommandAddSwitch(textureCommand, [Q_TEXTURE_SETTEXTURE, 
                                        "string", "integer"]);
  textureCommand = qCommandAddSwitch(textureCommand, [Q_TEXTURE_SETTEXTUREPOS, 
                                        "float", "float", "float", "float", "float", "integer"]);
  textureCommand = qCommandAddSwitch(textureCommand, [Q_TEXTURE_SLEEP, 
                                        "integer"]);
  textureCommand = qCommandAddSwitch(textureCommand, [Q_TEXTURE_DELETESCRIPT]);
}


 
initTextureCommand()
{
  buildTextureCommand();

  llListen(TEXTURE_COMMAND_CHANNEL, "", NULL_KEY, "");

  integer privateChannel = llGetStartParameter();
  if (privateChannel != 0)
    llListen(privateChannel, "", NULL_KEY, "");
}


 
executeTextureCommand(list switches)
{
  integer switchesLength = llGetListLength(switches);

  integer i;
  for (i = 1; i < switchesLength; )
    {
      string switch = llList2String(switches, i);

      if (switch == Q_TEXTURE_SETCOLOR)
	{
	  llSetColor(<llList2Float(switches, i+1),
		     llList2Float(switches, i+2),
		     llList2Float(switches, i+3)>, 
		     llList2Integer(switches, i+4));
	  i+=5;
	}
  
      else if (switch == Q_TEXTURE_SETALPHA)
	{
	  llSetAlpha(llList2Float(switches, i+1),
		     llList2Integer(switches, i+2));
	  i+=3;
	}

      else if (switch == Q_TEXTURE_SETTEXTURE)
	{
	  string texture = llList2String(switches, i+1);

	  if (texture == "blank")
	    texture = "5748decc-f629-461c-9a36-a35a221fe21f";

	  llSetTexture(texture,
		       llList2Integer(switches, i+2));
	  i+=3;
	}

      else if (switch == Q_TEXTURE_SETTEXTUREPOS)
	{
	  integer side = llList2Integer(switches, i+6);

	  llOffsetTexture(llList2Float(switches, i+1),
			  llList2Float(switches, i+2),
			  side);
	  llRotateTexture(llList2Float(switches, i+3),
			  side);
	  llScaleTexture(llList2Float(switches, i+4),
			 llList2Float(switches, i+5),
			 side);

	  i+=7;
	}

      else if (switch == Q_TEXTURE_DELETESCRIPT)
	{
	  llRemoveInventory(llGetScriptName());
	  i++;
	}


      else if (switch == Q_TEXTURE_SLEEP)
	{
	  llSleep(llList2Integer(switches, i+1));
	  i+=2;
	}


      else
	i = qCommandExecuteUniversalSwitch(switches, i);

    }
}


listenTextureCommand(integer channel, string name, key id,  string message)
{
  // we only accept commands on the public channel from our owner
  if ((channel == TEXTURE_COMMAND_CHANNEL) &&
      (name != llKey2Name(llGetOwner())))
    return;

  list switches = qCommandParseText(textureCommand, message);

  if (switches != [])
    executeTextureCommand(switches);
}


default
{
  state_entry()
    {
      initTextureCommand();
    }

  on_rez(integer n)
    {
      initTextureCommand();
    }


  listen(integer channel, string name, key id,  string message)
    {
      listenTextureCommand(channel, name, id, message);
    }
}
