#include "qLab/qCommand.lsl"
#include "qLab/qPosition.lsl"

#define TRANSFORM_COMMAND_CHANNEL 500

#define Q_TRANSFORM_SETGLOBALPOS    "-setGlobalPos"
#define Q_TRANSFORM_SETRELATIVEPOS  "-setRelativePos"
#define Q_TRANSFORM_SETSCALE        "-setScale"
#define Q_TRANSFORM_SETROT          "-setRot"
#define Q_TRANSFORM_DELETESCRIPT    "-deleteScript"
#define Q_TRANSFORM_SLEEP           "-sleep"
#define Q_TRANSFORM_GOTOGLOBALPOS   "-gotoGlobalPos"
#define Q_TRANSFORM_GOTORELATIVEPOS "-gotoRelativePos"
#define Q_TRANSFORM_GOTOSIM         "-gotoSim"


list transformCommand;

buildTransformCommand()
{
  transformCommand = qCommandCreate("transform");
  transformCommand = qCommandAddUniversalSwitches(transformCommand);

  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_SETGLOBALPOS, 
                                        "float", "float", "float"]);
  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_SETRELATIVEPOS, 
                                        "float", "float", "float"]);
  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_SETSCALE, 
                                        "float", "float", "float"]);
  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_SETROT, 
                                        "float", "float", "float"]);

  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_GOTOGLOBALPOS, 
                                        "float", "float", "float"]);
  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_GOTORELATIVEPOS, 
                                        "float", "float", "float"]);
  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_GOTOSIM, 
                                        "integer", "integer"]);
  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_SLEEP, 
                                        "integer"]);
  transformCommand = qCommandAddSwitch(transformCommand, [Q_TRANSFORM_DELETESCRIPT]);
}


 
initTransformCommand()
{
  buildTransformCommand();

  llListen(TRANSFORM_COMMAND_CHANNEL, "", NULL_KEY, "");

  integer privateChannel = llGetStartParameter();
  if (privateChannel != 0)
    llListen(privateChannel, "", NULL_KEY, "");
}


 
executeTransformCommand(list switches)
{
  integer switchesLength = llGetListLength(switches);

  integer i;
  for (i = 1; i < switchesLength; )
    {
      string switch = llList2String(switches, i);

      if (switch == Q_TRANSFORM_SETGLOBALPOS)
	{
	  qSetGlobalPosition(<llList2Float(switches, i+1),
			     llList2Float(switches, i+2),
			     llList2Float(switches, i+3)>);
	  i+=4;
	}
      else if (switch == Q_TRANSFORM_SETRELATIVEPOS)
	{
	  vector position = qGetGlobalPosition();

	  qSetGlobalPosition(position + (<llList2Float(switches, i+1),
					 llList2Float(switches, i+2),
					 llList2Float(switches, i+3)>));
	  i+=4;
	}
      else if (switch == Q_TRANSFORM_SETSCALE)
	{
	  llSetScale(<llList2Float(switches, i+1),
		     llList2Float(switches, i+2),
		     llList2Float(switches, i+3)>);
	  i+=4;
	}
      else if (switch == Q_TRANSFORM_SETROT)
	{
	  llSetRot(llEuler2Rot(<llList2Float(switches, i+1),
			       llList2Float(switches, i+2),
			       llList2Float(switches, i+3)>));
	  i+=4;
	}

      else if (switch == Q_TRANSFORM_DELETESCRIPT)
	{
	  llRemoveInventory(llGetScriptName());
	  i++;
	}


      else if (switch == Q_TRANSFORM_SLEEP)
	{
	  llSleep(llList2Integer(switches, i+1));
	  i+=2;
	}

      else if (switch == Q_TRANSFORM_GOTOSIM)
	{
	  vector position = qGetGlobalPosition();
	  position.x = llList2Float(switches, i+1) * 256 + 128;
	  position.y = llList2Float(switches, i+2) * 256 + 128;
	  qGotoGlobalPosition(position);

	  i+=3;
	}
      else if (switch == Q_TRANSFORM_GOTOGLOBALPOS)
	{
	  qGotoGlobalPosition(<llList2Float(switches, i+1),
			      llList2Float(switches, i+2),
			      llList2Float(switches, i+3)>);
	  i+=4;
	}
      else if (switch == Q_TRANSFORM_GOTORELATIVEPOS)
	{
	  vector position = qGetGlobalPosition();
	  qGotoGlobalPosition(position + (<llList2Float(switches, i+1),
					  llList2Float(switches, i+2),
					  llList2Float(switches, i+3)>));
	  i+=4;
	}

      else if (switch == Q_TRANSFORM_GOTOSIM)
	{
	  vector position = qGetGlobalPosition();
	  position.x = llList2Float(switches, i+1) * 256 + 128;
	  position.y = llList2Float(switches, i+2) * 256 + 128;
	  qGotoGlobalPosition(position);

	  i+=3;
	}



      else
	i = qCommandExecuteUniversalSwitch(switches, i);

    }
}


listenTransformCommand(integer channel, string name, key id,  string message)
{
  // we only accept commands on the public channel from our owner
  if ((channel == TRANSFORM_COMMAND_CHANNEL) &&
      (name != llKey2Name(llGetOwner())))
    return;

  list switches = qCommandParseText(transformCommand, message);

  if (switches != [])
    executeTransformCommand(switches);
}


default
{
  state_entry()
    {
      initTransformCommand();
    }

  on_rez(integer n)
    {
      initTransformCommand();
    }


  listen(integer channel, string name, key id,  string message)
    {
      listenTransformCommand(channel, name, id, message);
    }
}
